/*****************************************************************
 * Copyright (c) 2015 Texas Instruments and others
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Raymond Pang - Initial API and implementation
 *****************************************************************/

var gc = gc || {};
gc.databind = gc.databind  || {};

/** 
 */
gc.databind.AbstractMqttView = function( name, transportName ) 
{
	this.name = name;
	this.transport = gc.databind.CreateMqttTrasport( transportName );
	this.connected  = false;
};

/**
 */
gc.databind.AbstractMqttView.prototype._hasStatusBar = function()
{
	if ( this.statusbar === undefined )
	{
		this.statusbar = window.document.querySelector("ti-widget-statusbar");
	}
	return this.statusbar !== null;
}

	
/**
 */
gc.databind.AbstractMqttView.prototype.doLog = function ( msg )
{
	if ( this._hasStatusBar() )
	{
		this.statusbar.statusString1 = msg;
	}
};

/**
 */
gc.databind.AbstractMqttView.prototype.log = function ( msg )
{
	if ( msg && msg != "" )
	{
		this.doLog( msg );
	}
};

/**
 */
gc.databind.AbstractMqttView.prototype.logMsgArrival = function ( topic, data )
{
	var dt = new Date();
	this.log( dt.toLocaleTimeString()+ "|" + topic + "|" + JSON.stringify( data ) );
};

/**
 */
gc.databind.AbstractMqttView.prototype.setConnected = function ( connected )
{
	this.connected = connected;
	this.log( connected ? "Connected." : "Disconnected." );
	if ( this._hasStatusBar() )
	{
		this.statusbar.iconName = connected ? "ti-core-icons:link" : "ti-core-icons:nolink";
	}
};

/**
 */
gc.databind.AbstractMqttView.prototype.defaultProperties = function ()
{
};
	
/**
 */
gc.databind.AbstractMqttView.prototype.loadModelProperties = function ()
{
	var that = this;
	return gc.databind.registry.loadPropertiesFromFile('mqtt').then( function(properties)
          	{
              	if ( properties === undefined || properties[that.name] === undefined)
               	{
              		return that.defaultProperties();
                } 
               	return properties[that.name];
            })
            .catch( function(error)
            {
               	return that.defaultProperties();
			});   	
};
	
/**
 */
gc.databind.AbstractMqttView.prototype.saveModelProperties = function(properties)
{
	var that = this;
	return gc.databind.registry.loadPropertiesFromFile('mqtt').then( function(savedProperties)
          	{
				var newProperties = savedProperties;
              	if ( newProperties === undefined )
               	{
              		newProperties = {};
                } 
               	newProperties[that.name]  = properties;
               	newProperties['selected'] = that.name;
               	gc.databind.registry.savePropertiesToFile( undefined, { mqtt : newProperties } );
            });
};

/**
 */
gc.databind.AbstractMqttView.prototype.connect = function( prop)
{
}

/**
 */
gc.databind.AbstractMqttView.prototype.connectTransport = function( opt )
{
	var opt_onConnect = opt.onConnect;
	var opt_onFailure = opt.onFailure;
	var opt_onConnectionLost = opt.onConnectionLost;
	
	opt.onConnect = onConnect;
	opt.onFailure = onFailure;
	opt.onConnectionLost = onConnectionLost;
	
	if ( opt.onMessageDelivered === undefined )
	{
		opt.onMessageDelivered = onMessageDelivered;
	}
	this.log( this.transport.connect( opt ) );
	  
	var that = this;
	function onConnect() 
	{
	   	that.setConnected( true );
	   	if ( opt_onConnect )
	   		opt_onConnect();
	}
	  
	function onConnectionLost(error) 
	{
		that.log( error );
		that.setConnected( false );
	   	if ( opt_onConnectionLost )
	   		opt_onConnectionLost(error);
	}
	  
	function onFailure(error) 
	{
		that.log( error );
	   	if ( opt_onFailure )
	   		opt_onFailure(error);
	}

	function onMessageDelivered() 
	{
	}
};
	
/**
 */
gc.databind.AbstractMqttView.prototype.disconnect = function()
{
	if ( this.connected )
	{
		this.setConnected(false);
		this.log( "Disconnecting..." );
		this.log( this.transport.disconnect() );	
	}
};
	
/**
 */
gc.databind.AbstractMqttView.prototype.subscribe = function( topic, opt )
{
	if ( this.connected )
	{
		var that = this;
		function onFailure(error) 
		{
			that.log( error );
		}
		opt = opt || { timeout: 3, qos: 0 }
		opt.onFailure = onFailure;
		this.log( this.transport.subscribe( topic, opt ) );	
	}
};
	
/**
 */
gc.databind.AbstractMqttView.prototype.unsubscribe = function( topic, opt )
{
	if ( this.connected )
	{
		var that = this;
		function onFailure(error) 
		{
			that.log( error );
		}
		if ( opt === undefined )
		{
			opt = { timeout: 3, qos: 0 };
		}
		opt.onFailure = onFailure;
		this.log( this.transport.unsubscribe( topic, opt ) );
	}
};
	
/**
 */
gc.databind.AbstractMqttView.prototype.publish = function( topic, payload, opt )
{
	if ( this.connected )
	{
		this.log( this.transport.publish( topic, payload, opt ) );
	}
};

/**
 */
gc.databind.AbstractMqttView.prototype.sendTopic = function( name, value )
{
	this.log( "Publishing is not supported by the broker." );      		
};

/**
 */
gc.databind.AbstractMqttView.prototype.setTopic = function( name, value )
{
	var topic = gc.databind.registry.getBinding( 'mqtt.' + name );
	if (topic )
	{
		topic.setData( value );
	  	topic.setStale(false);
	}
}

/**
 */
gc.databind.AbstractMqttView.prototype.isMacro = function( name )
{
	return ( name.length > 2 && name[0] === '$' && name[name.length-1] === '$')
}

/**
 */
gc.databind.AbstractMqttView.prototype.replaceMacro = function( name )
{
	if ( this.searchArgs === undefined )
	{
		this.searchArgs = {};
		var queries = window.location.search.replace(/^\?/, '').split('&');
	    for( var i = 0; i < queries.length; i++ ) {
	        var split = queries[i].split('=');
	        this.searchArgs[split[0]] = split[1];
	    }		
	}
	var macro = name.substr( 1, name.length - 2 );
	return ( this.searchArgs[macro] === undefined ? name : this.searchArgs[macro] );
}

/**
 */
gc.databind.AbstractMqttView.prototype.substitueMacros = function( prop )
{
	
	for( var i in prop.devices )
	{
		if ( this.isMacro( prop.devices[i].id ))
		{
			prop.devices[i].id = this.replaceMacro( prop.devices[i].id );
		}
	}
}

/**
 */
gc.databind.AbstractMqttView.prototype.ready = function( )
{
	var that = this;
	this.loadModelProperties().then( function(prop) {
   		that.substitueMacros( prop );
		that.connect( prop );
	});
    gc.databind.MqttModel.setMqttViewInstance( this );
}

/**
 */
gc.databind.AbstractMqttView.prototype.retire = function( )
{
	this.disconnect();
}

/**
 */
gc.databind.AbstractMqttView.prototype.numberOfDotInTopicName = function(  )
{
	return 1;
}

/**
 */
gc.databind.AbstractMqttView.prototype.isTopicName = function( name )
{
	return name.split('.').length == this.numberOfDotInTopicName();
	
}


/*****************************************************************/


/** 
 */
gc.databind.AbstractMqttDesignView = function( name, dialog, transportName ) 
{
	gc.databind.AbstractMqttView.call( this, name, transportName );
	this.dialog = dialog;
	this.bindingEndpoints = [];
	
	var that = this;
	this.onCommit = function ( event, detail, sender ) {
		var prop =  that.extractPropertiesFromDialog();
		that.saveModelProperties( prop );
		that.updateBindingEndpoints( prop.devices );
		that._cachedProp = null;
  	};
  	
	this.onCancel = function ( event, detail, sender ) {
		that.disconnect();
		that._cachedProp = null;
 	};
  	
    this.onSetConnection = function ( event, detail, sender ) {
		if ( event.detail.connected === true )
		{
			var prop = that.extractPropertiesFromDialog();
   			that.connect(prop);
		}
		else
		{
			that.disconnect();
		}
	};
	
	this.onSendCommand = function ( event, detail, sender ) {
		var args = event.detail.args;
		if ( args.length >= 2 )
		{
			var msg = args[args.length-1];
			var topic = args[0];
			for( var i = 1; i < args.length-1; i++  )
			{
				topic += "." + args[i];
			}
			that.sendTopic( topic, msg );
		}
	};
};

gc.databind.AbstractMqttDesignView.prototype = new gc.databind.AbstractMqttView();

/**
 */
gc.databind.AbstractMqttDesignView.prototype.open = function ()
{
	return this.dialog.open();
};
	
/**
 */
gc.databind.AbstractMqttDesignView.prototype.close = function ()
{
	return this.dialog.close();
};
	
/**
 */
gc.databind.AbstractMqttDesignView.prototype.doLog = function ( msg )
{
	gc.databind.AbstractMqttView.prototype.doLog.call( this, msg );
	this.dialog.addConsoleMessage( msg );
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.setConnected = function ( connected )
{
	gc.databind.AbstractMqttView.prototype.setConnected.call( this, connected );
    this.dialog.set('connection.connected', connected);
    ;
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.createDialogRow = function( rowProperties )
{
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.extractFromDialogRow = function( row )
{
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.initializeDialog = function( properties )
{
    this.dialog.set('connection.connected',false);
   this.dialog.set('devices.list',[]);
 	for( var i in properties.devices )
 	{
 		this.dialog.splice('devices.list',
			this.dialog.devices.list.length, 
			0, 
			this.createDialogRow( properties.devices[i] ) 
 		);
	};		
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.extractPropertiesFromDialog = function()
{
	var data = [];
	for( var i in this.dialog.devices.list )
	{
		data.splice( data.length, 0, this.extractFromDialogRow( this.dialog.devices.list[i] ) );
	}
	return data;
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.updateBindingEndpoints = function ( topics )
{
	var that = this;
	this.bindingEndpoints = [];
     		
	var helper = function ( prepend, name, children )
	{
		that.bindingEndpoints.splice( that.bindingEndpoints.length, 0, prepend + name);
   		if( typeof children == "object" )
   		{
   			for( var child in children )
   			{
   				if ( prepend.length == 0 )
   					helper( name + ".", child, children[child] ); 
   				else
           			helper( prepend + name + ".", child, children[child] );           					
   			}
   		}
	};
         	
	for ( var i in topics )
		helper( "", topics[i].name, topics[i].events );

	this.bindingEndpoints.sort();
   			
	var last_i;
	for ( var i = 0; i < this.bindingEndpoints.length; i++ )
		if (( last_i = this.bindingEndpoints.lastIndexOf( this.bindingEndpoints[i]) ) !== i)
			this.bindingEndpoints.splice( i + 1, last_i - i );
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.matchBindingEndpoints = function ( prefix )
{
	var list = [];
	prefix = prefix || "";
	var lastdot = prefix.lastIndexOf(".");
	for ( var i = 0; i < this.bindingEndpoints.length; i++ )
	{
	   	var name = this.bindingEndpoints[i];
	   	if ( name.indexOf( prefix ) == 0 && name.lastIndexOf( ".") == lastdot )
	   	{
	   		if ( lastdot !== -1)
    			name = name.substr( lastdot + 1 );
 	    	list.splice( list.length, 0, name );
    	}
	}
    list.sort();
	return list;
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.disconnect = function()
{
	if ( this.connected )
	{
		for ( var i in this.dialog.devices.list )
		{
	       	this.dialog.set('devices.list.'+i+'.active', false);
		}
		gc.databind.AbstractMqttView.prototype.disconnect.call( this );
	}
};
	
/**
 */
gc.databind.AbstractMqttDesignView.prototype.addEventListeners = function( )
{
	this.dialog.addEventListener( 'commit', this.onCommit );
	this.dialog.addEventListener( 'cancel', this.onCancel );
    this.dialog.addEventListener( 'setConnection', this.onSetConnection );
	this.dialog.addEventListener( 'sendCommand', this.onSendCommand );
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.removeEventListeners = function( )
{
	this.dialog.removeEventListener( 'commit', this.onCommit );
	this.dialog.removeEventListener( 'cancel', this.onCancel );
    this.dialog.removeEventListener( 'setConnection', this.onSetConnection );
	this.dialog.removeEventListener( 'sendCommand', this.onSendCommand );
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.ready = function( )
{
	this.addEventListeners();
	
	var that = this;
	var startUp = function( prop )
	{
   		that.initializeDialog( prop );
   		that.updateBindingEndpoints( prop.devices );
	};
	
	if ( this._cachedProp )
	{
		startUp(this._cachedProp);
	}
	else
	{
		this.loadModelProperties().then( function(prop) {
			startUp(prop);
		});
	}
    gc.databind.MqttModel.setMqttViewInstance( this );
};

/**
 */
gc.databind.AbstractMqttDesignView.prototype.retire = function( )
{
	this._cachedProp = this.extractPropertiesFromDialog();
	this.disconnect();
	this.removeEventListeners();
};

